'''
$Log: clickyLibTest.py,v $
Revision 1.15  2005/10/30 06:26:46  jangc
shell mode is added

Revision 1.14  2005/06/26 07:42:56  jangc
setUrl method is added. link is added to file, folder skin

Revision 1.13  2005/06/19 07:32:14  jangc
download action is added

Revision 1.12  2005/04/10 03:10:50  jangc
minor changes

Revision 1.11  2005/03/27 02:08:50  jangc
create folder, file actions are added

Revision 1.10  2005/03/14 15:44:28  jangc
unzip̶   ɵ ߰.

Revision 1.9  2005/03/13 11:47:33  jangc
some tests are added

Revision 1.8  2005/03/13 11:09:05  jangc
copy testcode added

Revision 1.7  2005/03/13 07:13:09  jangc
rename testcode is added

Revision 1.6  2005/03/13 04:09:39  jangc
file move testcode addded

Revision 1.5  2005/03/13 03:36:09  jangc
remove method are added to File, Folder Class

'''
import unittest
import shutil
import os

from clickyLib.clickyLib import *
import clickyLib.exceptionLib
from clickyLib import config

class FileClassTestCase(unittest.TestCase):
    def setUp(self):
        try: shutil.rmtree('testDirectory')
        except: pass
        try: shutil.rmtree('testDirectory2')
        except: pass
        try: os.mkdir('testDirectory')
        except: pass
        try: os.mkdir('testDirectory2')
        except: pass
        for num in range(10):
            file('testDirectory/testFile%d' % num, 'w').write(str(num))

    def test_Init(self):
        path = 'testDirectory/testFile0'
        fObj = File(path)
        self.assert_(fObj)
        self.assertEquals(fObj.name, 'testFile0')
        self.assert_(os.path.exists(path))
        self.assert_(fObj.atime)
        self.assert_(fObj.mtime)
        self.assert_(fObj.fileSize)

    def test_Move(self):
        srcPath = 'testDirectory/testFile0'
        dstPath = 'testDirectory2/testFile0'
        fObj = File(srcPath)
        fObj.moveTo('testDirectory2')
        
        self.assert_(os.path.exists(dstPath))
        self.assertEquals(os.path.realpath(fObj.path), os.path.realpath(dstPath))
        self.failIf(os.path.exists(srcPath))

    def test_IllegalMove(self):
        # ڱ ڽ ġ ϴ 
        orgPath = 'testDirectory/testFile0'
        fObj = File(orgPath)
        self.assertRaises(exceptionLib.IllegalDestination, fObj.moveTo, 'testDirectory')
        
        self.assert_(os.path.exists(orgPath))
        self.assertEquals(fObj.path, os.path.realpath(orgPath))

        #  ʴ ġ ̵ 
        orgPath = 'testDirectory/testFile1'
        fObj = File(orgPath)
        self.failIf(os.path.exists(os.path.realpath('notExistsDirectory')))
        self.assertRaises(exceptionLib.IllegalDestination, fObj.moveTo, 'notExistsDirectory')
        
        self.assert_(os.path.exists(orgPath))
        self.failIf(os.path.exists('notExistsDirectory'))
        self.assertEquals(fObj.path, os.path.realpath(orgPath))


        #  ̸  ̹ ϴ 
        orgPath = 'testDirectory/testFile2'
        fObj = File(orgPath)
        fObj.moveTo('testDirectory2')

        file('testDirectory/testFile2', 'w').write('2')
        fObj = File(orgPath)
        self.assertRaises(exceptionLib.IllegalDestination, fObj.moveTo, 'testDirectory2')

    def test_Rename(self):
        fObj = File('testDirectory/testFile0')
        self.assert_(fObj.rename('NewName'))
        
        self.assert_(os.path.exists('testDirectory/NewName'))
        self.assertEquals(fObj.name, 'NewName')
        self.assertEquals(os.path.realpath(fObj.path), os.path.realpath('testDirectory/NewName'))
        self.failIf(os.path.exists('testDirectory/testFile0'))

        self.assert_(fObj.rename('ѱ'))
        self.assert_(os.path.exists('testDirectory/ѱ'))

        fObj = File('testDirectory/testFile2')
        self.failIf(fObj.rename('testFile3'))
        self.assert_(os.path.exists('testDirectory/testFile2'))
        
        fObj = File('testDirectory/testFile3')
        self.failIf(fObj.rename('testDirectory2/testFile3'))
        self.assert_(os.path.exists('testDirectory/testFile3'))
        
    def test_Copy(self):
        fObj = File('testDirectory/testFile0')
        orgPath = fObj.path
        self.assert_(fObj.copyTo('testDirectory2'))
        self.assert_(os.path.exists('testDirectory2/testFile0'))
        self.assert_(os.path.exists('testDirectory/testFile0'))
        self.assertEquals(fObj.path, orgPath)

        fObj = File('testDirectory/testFile2')
        orgPath = fObj.path
        self.assertRaises(IOError, fObj.copyTo, 'testDirectory2/testFile2')
        self.assert_(os.path.exists('testDirectory/testFile2'))
        self.failIf(os.path.exists('testDirectory2/testFile2'))
        self.assertEquals(fObj.path, orgPath)

        fObj = File('testDirectory/testFile0')
        orgPath = fObj.path
        self.assert_(os.path.exists('testDirectory2/testFile0'))
        self.assertRaises(exceptionLib.IllegalDestination, fObj.copyTo, 'testDirectory2')
        self.assertEquals(fObj.path, orgPath)
        
    def test_Remove(self):
        fObj = File('testDirectory/testFile0')
        self.assert_(fObj.remove())
        self.failIf(os.path.exists('testDirectory/testFile0'))

        fObj = File('testDirectory2')
        self.failIf(fObj.remove())
        self.assert_(os.path.exists('testDirectory2'))
    
    def test_Write(self):
        fObj = File('testDirectory/testFile0') 
        text1 = 'Clicky is good program'
        fObj.write(text1)
        text2 = file('testDirectory/testFile0').read()
        self.assertEquals(text1, text2)

    def test_Read(self):
        text = 'This is testcode'
        file('testDirectory/testFile0', 'w').write(text)
        fObj = File('testDirectory/testFile0')
        self.assertEquals(fObj.read(), text)

    def test_setUrl(self): # FIXME ׽Ʈ ߰ϰ...urlMap order dict ĥ .
        config.urlMap = {os.getcwd():'http://jangc.net/clicky'} # for test
        fObj = File('testDirectory/testFile8')
        self.assertEquals(fObj.url, 'http://jangc.net/clicky/testDirectory/testFile8')
        self.assertEquals(fObj.setUrl(), 'http://jangc.net/clicky/testDirectory/testFile8')
        #self.assertEquals(fObj.setUrl(), '
        config.urlMap = {'/home/hosting_users/bluejan/www/':'http://jangc.net/' }
        fObj = File('/home/hosting_users/bluejan/www/zboard/') 
        self.assertEquals(fObj.url, 'http://jangc.net/zboard/')

        
    def tearDown(self):
        try: shutil.rmtree('testDirectory')
        except: pass
        try: shutil.rmtree('testDirectory2')
        except: pass

class FolderClassTestCase(unittest.TestCase):
    def setUp(self):
        try: shutil.rmtree('testDirectory')
        except: pass
        try: os.mkdir('testDirectory')
        except: pass
        try: os.mkdir('testDirectory2')
        except: pass
        for num in range(10):
            file('testDirectory/testFile%d' % num, 'w').write(str(num))
            os.mkdir('testDirectory/testDirectory%d' % num)

    def test_FolderObjInit(self):
        path = 'testDirectory/testDirectory0'
        folder = Folder(path)
        self.assert_(folder)
        self.assertEquals(folder.path, os.path.realpath(path)) 
        self.assertEquals(folder.name, 'testDirectory0')
        self.assert_(folder.atime)
        self.assert_(folder.mtime)
        self.assert_(folder.fileSize)

    def test_FolderObjMove(self):
        # 1
        fObj = Folder('testDirectory/testDirectory0')
        fObj.moveTo('testDirectory/testDirectory1')
        dstPath = 'testDirectory/testDirectory1/testDirectory0'
        self.assert_(os.path.exists(dstPath))
        self.assertEquals(os.path.realpath(fObj.path), os.path.realpath(dstPath))

    def test_FolderObjCopy(self):
        fObj = Folder('testDirectory/testDirectory0')
        orgPath = fObj.path
        self.assert_(fObj.copyTo('testDirectory2'))
        self.assert_(os.path.exists('testDirectory/testDirectory1'))
        self.assert_(os.path.exists('testDirectory2/testDirectory0'))
        self.assertEquals(fObj.path, orgPath)

        fObj = Folder('testDirectory/testDirectory0')
        self.assertRaises(exceptionLib.IllegalDestination, fObj.copyTo, 'testDirectory')
        self.assert_(os.path.exists('testDirectory/testDirectory0'))

        fObj = Folder('testDirectory/testDirectory0')
        self.failIf(os.path.exists('notExistsDirectory'))
        self.assertRaises(OSError, fObj.copyTo, 'notExistsDirectory')
        
        fObj = Folder('testDirectory/testDirectory0')
        self.assert_(os.path.exists('testDirectory2'))
        self.assertRaises(exceptionLib.IllegalDestination, fObj.copyTo, 'testDirectory2')
        
        fObj = Folder('testDirectory')
        self.assertRaises(exceptionLib.IllegalDestination, fObj.copyTo, 'testDirectory/testDirectory0')
        self.assert_(os.path.exists('testDirectory'))
        self.failIf(os.path.exists('testDirectory/testDirectory0/testDirectory'))

        fObj = Folder('testDirectory')
        self.assert_(fObj.copyTo('testDirectory2'))
        self.assert_(os.path.exists('testDirectory2/testDirectory'))
        self.assert_(os.path.exists('testDirectory'))
        fnameList1 = os.listdir(fObj.path)
        fnameList1.sort()
        fnameList2 = os.listdir('testDirectory2/testDirectory')
        fnameList2.sort()
        self.assertEquals(fnameList1, fnameList2)


        
        
    def test_FolderObjIllegalMove(self):
        # ڱ ڽ ִ ġ ̵
        fObj = Folder('testDirectory/testDirectory3')
        orgPath = fObj.path
        self.assertRaises(exceptionLib.IllegalDestination, fObj.moveTo, 'testDirectory')
        self.assert_(os.path.exists('testDirectory/testDirectory3'))
        self.assertEquals(fObj.path, orgPath)

        # ڱ ڽ   ̵
        fObj = Folder('testDirectory/testDirectory4')
        orgPath = fObj.path
        self.assertRaises(exceptionLib.IllegalDestination, fObj.moveTo, 'testDirectory/testDirectory4')
        self.assert_(os.path.exists('testDirectory/testDirectory4'))
        self.failIf(os.path.exists('testDirectory/testDirectory4/testDirectory4'))
        self.assertEquals(fObj.path, orgPath)
        
        #    ʴ  ̵
        fObj = Folder('testDirectory/testDirectory5')
        orgPath = fObj.path
        self.assertRaises(exceptionLib.IllegalDestination, fObj.moveTo, 'notExistsDirectory/testDirectory5')
        self.assert_(os.path.exists('testDirectory/testDirectory5'))
        self.failIf(os.path.exists('notExistsDirectory/testDirectory5'))
        self.assertEquals(fObj.path, orgPath)
 
    def test_FolderObjRemove(self):
        fObj = Folder('testDirectory/testDirectory0')
        self.assert_(fObj.remove())
        self.failIf(os.path.exists('testDirectory/testDirectory0'))

        fObj = Folder('testDirectory/testFile0')
        self.failIf(fObj.remove())
        self.assert_(os.path.exists('testDirectory/testFile0'))

        fObj = Folder('testDirectory')
        self.assert_(fObj.remove())
        self.failIf(os.path.exists('testDirectory'))
        

    def tearDown(self):
        try: shutil.rmtree('testDirectory')
        except: pass
        try: shutil.rmtree('testDirectory2')
        except: pass

class DirectoryClassTestCase(unittest.TestCase):
    def setUp(self):
        try: shutil.rmtree('testDirectory')
        except: pass
        try: os.mkdir('testDirectory')
        except: pass

        for num in range(10):
            file('testDirectory/testFile%d' % num, 'w').write(str(num))
            os.mkdir('testDirectory/testDirectory%d' % num)

    def test_DirectoryObjInit(self):
        path = os.path.join(os.getcwd(), 'testDirectory')
        directory = Directory(path)
        self.assertEquals(directory.path, path)
        self.assertEquals(len(directory.fileNameList), 10)
        self.assertEquals(len(directory.folderNameList), 10)
        self.assertEquals(len(directory), 20)
        for fObj in directory:
            self.assert_(isinstance(fObj, File) or isinstance(fObj, Folder)) 

    def test_getUpstairsPath(self):
        path = os.path.join(os.getcwd(), 'testDirectory')
        directory = Directory(path)
        upStairPath = directory.getUpstairsPath()
        self.assertEquals(upStairPath, os.getcwd())

    def tearDown(self):
        try: shutil.rmtree('testDirectory')
        except: pass

class ETCTestCase(unittest.TestCase):
    def setUp(self):
        try: shutil.rmtree('testDirectory')
        except: pass
        try: os.mkdir('testDirectory')
        except: pass

        for num in range(10):
            file('testDirectory/testFile%d' % num, 'w').write(str(num))
            os.mkdir('testDirectory/testDirectory%d' % num)

    def test_getFileOrFolderInstanceByPath(self): # getFileOrFolderByPath == getFObject
        folderPath = 'testDirectory/testDirectory1'
        fObj = getFileOrFolderInstanceByPath(folderPath)
        self.assert_(isinstance(fObj, Folder))

        filePath = 'testDirectory/testFile1'
        fObj = getFileOrFolderInstanceByPath(filePath)
        self.assert_(isinstance(fObj, File))

        notExistsPath = 'ugvjkfjkfjhtuajkadgweru'
        fObj = getFileOrFolderInstanceByPath(notExistsPath)
        self.failIf(fObj)

    def test_isUpperFolder(self):
        path1 = '/abc'
        path2 = '/abc/def'
        self.assert_(isUpperFolder(path1, path2))

        path1 = '/abc/def'
        path2 = '/abc/defg'
        self.failIf(isUpperFolder(path1, path2))
    
    def test_convertPlainIntoHTML(self):
        # 1
        plainText = '''Hello, World.
My Name Is Clicky.'''
        ret = convertPlainIntoHTML(plainText)
        self.assertEquals(ret, '''Hello, World.<br/>\nMy Name Is Clicky.''')
        
        # 2 
        plainText = '''<Hello, World.
My Name Is Clicky.>'''
        ret = convertPlainIntoHTML(plainText)
        self.assertEquals(ret, '''&lt;Hello, World.<br/>\nMy Name Is Clicky.>''')
        
        # 3 
        plainText = '''&lt;Hello, World.
My Name Is Clicky.>'''
        ret = convertPlainIntoHTML(plainText)
        self.assertEquals(ret, '''&amp;lt;Hello, World.<br/>\nMy Name Is Clicky.>''')

        # 4 
        plainText = '''"Hello, World.
My Name Is Clicky."'''
        ret = convertPlainIntoHTML(plainText)
        self.assertEquals(ret, '''&quot;Hello, World.<br/>\nMy Name Is Clicky.&quot;''')
        
        # 4
        plainText = '''Hello,     World.
My Name Is Clicky.'''
        ret = convertPlainIntoHTML(plainText)
        self.assertEquals(ret, '''Hello, &nbsp;&nbsp;&nbsp;&nbsp;World.<br/>\nMy Name Is Clicky.''')

    def test_bin(self):
        self.assertEquals(bin(10, 4), '1010')
        self.assertEquals(bin(10, 6), '001010')
    
        self.assertEquals(bin(10, 1), '1010') 

        self.assertEquals(bin(1, 6), '000001')

    def test_createFile(self):
        self.assert_(createFile('testDirectory', 'createdFile', 'contents')) 
        self.assert_(os.path.exists('testDirectory/createdFile'))
    
        self.assert_(createFile('testDirectory', 'createdFile2', '')) 
        self.assert_(os.path.exists('testDirectory/createdFile'))

        self.failIf(createFile('testDirectory', './createdFile3', 'contents')) 
        self.failIf(os.path.exists('testDirectory/createdFile3'))

        self.failIf(createFile('testDirectory', 'created?File4', 'contents')) 
        self.failIf(os.path.exists('testDirectory/created?File4'))

    def test_createFolder(self):
        self.assert_(createFolder('testDirectory', 'createdFolder')) 
        self.assert_(os.path.exists('testDirectory/createdFolder'))
    
        self.failIf(createFolder('testDirectory', './createdFolder2')) 
        self.failIf(os.path.exists('testDirectory/createdFolder2'))

        self.failIf(createFolder('testDirectory', 'created?Folder3')) 
        self.failIf(os.path.exists('testDirectory/created?Folder3'))

    def tearDown(self):
        try: shutil.rmtree('testDirectory')
        except: pass
    
if __name__ == '__main__':
    unittest.main(argv=('', '-v'))
