import re
import urllib
import urllib2
import urlparse
from StringIO import StringIO
import htmllib, formatter
import ClientCookie
import ClientForm
from ClientCookie import HTTPRedirectHandler
from ClientCookie import _urllib2_support

class MyHTTPRedirectHandler(HTTPRedirectHandler):
    def redirect_request(self, newurl, req, fp, code, msg, headers):
        self.newurl = newurl
        return HTTPRedirectHandler.redirect_request(self, newurl, req, fp, code, msg, headers)

redirectHandler = MyHTTPRedirectHandler()

def urlopen(url, data=None):
    if _urllib2_support._opener is None:
        _urllib2_support.urlopen_lock.acquire()
        try:
            if _urllib2_support._opener is None:
                _urllib2_support._opener = _urllib2_support.build_opener(redirectHandler)
        finally:
            _urllib2_support.urlopen_lock.release()
    return _urllib2_support._opener.open(url, data)

class _FormParser(ClientForm._AbstractFormParser, htmllib.HTMLParser):
    # This is still here for compatibility with Python 1.5.2.
    # It doesn't do the right thing with XHTML.
    def __init__(self, ignore_errors, entitydefs=None):
        htmllib.HTMLParser.__init__(self, formatter.NullFormatter())
        ClientForm._AbstractFormParser.__init__(self, ignore_errors, entitydefs)

    def do_option(self, attrs):
        ClientForm._AbstractFormParser._start_option(self, attrs)

class WebObject:
    defaultHeaders = {'Connection':'close', 'Accept language':'ko'}
    def __init__(self, url=''):
        self.response = None
        self.body = ''
        self.baseurl = ''
        self.encode = ''
        if url:
            self.click(url)

    def makeHeader(self, headers):
        if headers == None:
            headers = {}
        requestHeaders = self.defaultHeaders.copy()
        requestHeaders.update(headers)
        return requestHeaders

    def getResponse(self):
        return self.response

    def parseForm(self, html, response):
        try:
            self.forms = ClientForm.ParseFile(StringIO(html), response.geturl())
        except:
            orgParser = ClientForm._FORM_PARSER_CLASS
            ClientForm._FORM_PARSER_CLASS = _FormParser
            self.forms = ClientForm.ParseFile(StringIO(html), response.geturl())
            ClientForm._FORM_PARSER_CLASS = orgParser
        return self.forms

    def parseFrame(self):
        self.frames = {}   # key = nameattr or index, value = WebObject()
        i = 0
        for frame in self.getTagList('frame'):
            url = frame.attrs['src']
            url = urlparse.urljoin(self.baseurl, url)
            frameName = frame.attrs.get('name', '') or i
            self.frames[frameName] = self.__class__(url)
            i += 1
        return self.frames

    def getMetaTagRedirectionUrl(self):
        for meta in self.getTagList('meta', {'http-equiv':'Refresh'}):
            m = re.search(r'URL=(\S+)', meta.attrs['content'], re.I)
            if m:
                url = m.group(1)
                url = urlparse.urljoin(self.baseurl, url)
                return url

    def setEncodingCharacterSet(self, encode):
        self.encode = encode

    def textEncoding(self):
        if self.encode:
            meta = self.getTagByAttribute('meta', **{'http-equiv':'Content-Type'})
            if meta:
                m = re.search(r'charset=(\S+)', meta.attrs['content'], re.I)
                if m and m.group(1) != self.encode:
                    self.body = unicode(self.body, m.group(1)).encode(self.encode)

    def setResponse(self, response):
        self.response = response
        self.body = self.response.read()

        url = self.getMetaTagRedirectionUrl()
        if url:
            return self.click(url)
        self.textEncoding()
        return response
    
    def getTitle(self):
        titleTags = self.getTagList('title')
        if titleTags:
            return titleTags[0].data
        return None

    def getBody(self):
        return self.body

    #---------------------------------------------------
    # clicks
    def click(self, url, form=None, headers=None):
        if url.__class__ == Tag and url.name == 'a':
            url = url.attrs['href']
        elif url.__class__ == Tag and url.name == 'frame':
            url = url.attrs['src']
        elif url.__class__ == ClientForm.HTMLForm:
            return self.submit(url, headers)
        self.baseurl = urlparse.urljoin(self.baseurl, url)
        url = self.baseurl
        if form != None:
            form = urllib.urlencode(form)
        data = form
        requestHeaders = self.makeHeader(headers)
        request = urllib2.Request(url, data, requestHeaders)
        redirectHandler.newurl = ''
        response = urlopen(request)
        if redirectHandler.newurl:
            self.baseurl = redirectHandler.newurl
        self.setResponse(response)
        return self

    clickUrl = click

    def submit(self, formObj, headers=None):
        self.baseurl = formObj.action
        if formObj.enctype != 'application/x-www-form-urlencoded':
            formObj.enctype = 'application/x-www-form-urlencoded'
        header = self.makeHeader(headers)
        request = formObj.click()
        request.headers.update(header)
        redirectHandler.newurl = ''
        response = urlopen(request)
        if redirectHandler.newurl:
            self.baseurl = redirectHandler.newurl
        self.setResponse(response)
        return self

    def clickByName(self, name):
        link = self.getTagByText(tagName='a', text=name)
        return self.click(link)

    def clickBySubstring(self, s):
        alist = self.getTagList('a')
        for a in alist:
            if a.data.find(s) > -1:
                return self.click(a)
        raise KeyError, s

    def clickForm(self, formName='', formOrder=0, **kw):
        if formName:
            form = self.getFormByAttribute(name=formName)
        else:
            form = self.getFormByOrder(formOrder)
        for key, value in kw.items():
            form[key] = value
        return self.submit(form)

    def clickFrame(self, **kw):
        frame = self.getTagByAttribute(tagName='frame', **kw)
        return self.click(frame)

    #---------------------------------------------------
    def getTagList(self, name, kw=None):
        parser = TagParser(name, kw)
        parser.feed(self.body)
        return parser.tagList
    
    def getTagListByAttrSubstring(self, tagName, **kw):
        tagList = self.getTagList(tagName)
        L = []
        for tag in tagList:
            for key, value in kw.items():
                if not (tag[key].find(value) >= 0):
                    break
            else:
                L.append(tag)
        return L    

    def getTagByText(self, tagName, text):
        links = self.getTagList(tagName)
        for a in links:
            if a.data == text:
                return a
        raise KeyError, text

    def getTagByOrder(self, tagName, n):
        links = self.getTagList(tagName)
        return links[n]

    def getTagByAttribute(self, tagName, **kw):
        objList = self.getTagList(tagName)
        for obj in objList:
            for k, v in kw.items():
                if obj.attrs[k] != v:
                    break
            else:
                return obj
        raise KeyError, '%s' % kw.keys()

    #---------------------------------------------------
    # forms
    def getFormByOrder(self, n):
        forms = self.parseForm(self.body, self.response)
        return forms[n]

    def getFormByAttribute(self, **kw):
        forms = self.parseForm(self.body, self.response)
        for form in self.forms:
            for key, value in kw.items():
                if getattr(form, key) != value:
                    break
            else:
                return form

class Tag:
    def __init__(self, name, attrs):
        self.name = name
        if type(attrs) == type({}):
            self.attrs = attrs
        else:
            self.attrs = dict(attrs)
        self.data = ''
    def getName(self):
        return self.name
    def setData(self, data):
        self.data = data
    def __nonzero__(self):
        return True
    def __getitem__(self, name):
        if hasattr(self, name):
            return getattr(self, name)
        return self.attrs[name]
    def __repr__(self):
        return 'tag name=%s attrs=%s data=%s' % (self.name, self.attrs, self.data)

class TagParser(htmllib.HTMLParser):
    def __init__(self, tagName, kw=None):
        htmllib.HTMLParser.__init__(self, formatter.NullFormatter())
        self.tagList = []
        self.tagName = tagName
        self.working = 0
        self.kw = kw

    def start_frame(self, attrs):
        pass

    def end_frame(self, attrs):
        pass

    def handle_starttag(self, tag, method, attrs):
        if tag == self.tagName:
            if self.kw:
                for key, value in self.kw.items():
                    if (key, value) not in attrs:
                        break
                else:
                    self.tagList.append(Tag(tag, attrs))
                    self.working += 1
            else:
                self.tagList.append(Tag(tag, attrs))
                self.working += 1

    def handle_endtag(self, tag, method):
        if tag == self.tagName:
            self.working -= 1

    def handle_data(self, data):
        if self.working:
            self.tagList[-1].data += data
